import { Module, Provider } from '@nestjs/common';
import { CqrsModule } from '@nestjs/cqrs';
import {
  AD_MESSAGE_PUBLISHER,
  AD_REPOSITORY,
  PARAMS_PROVIDER,
  TIMEZONE_FINDER,
} from './ad.di-tokens';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { AdRepository } from './infrastructure/ad.repository';
import { PrismaService } from './infrastructure/prisma.service';
import { DefaultParamsProvider } from './infrastructure/default-params-provider';
import { TimezoneFinder } from './infrastructure/timezone-finder';
import { AdMapper } from './ad.mapper';

const mappers: Provider[] = [AdMapper];

const repositories: Provider[] = [
  {
    provide: AD_REPOSITORY,
    useClass: AdRepository,
  },
];

const messagePublishers: Provider[] = [
  {
    provide: AD_MESSAGE_PUBLISHER,
    useExisting: MessageBrokerPublisher,
  },
];
const orms: Provider[] = [PrismaService];

const adapters: Provider[] = [
  {
    provide: PARAMS_PROVIDER,
    useClass: DefaultParamsProvider,
  },
  {
    provide: TIMEZONE_FINDER,
    useClass: TimezoneFinder,
  },
];

@Module({
  imports: [CqrsModule],
  providers: [
    ...mappers,
    ...repositories,
    ...messagePublishers,
    ...orms,
    ...adapters,
  ],
  exports: [
    PrismaService,
    AdMapper,
    AD_REPOSITORY,
    PARAMS_PROVIDER,
    TIMEZONE_FINDER,
  ],
})
export class AdModule {}
